from django_select2.forms import (
    Select2Widget,
    HeavySelect2Widget,
    Select2MultipleWidget,
    HeavySelect2MultipleWidget,
    HeavySelect2TagWidget,
    ModelSelect2Widget,
    ModelSelect2MultipleWidget,
    ModelSelect2TagWidget,
)

from .widgets import AddAnotherMixin


class Select2AddAnother(AddAnotherMixin, Select2Widget):
    pass

class HeavySelect2AddAnother(AddAnotherMixin, HeavySelect2Widget):
    pass

class Select2MultipleAddAnother(AddAnotherMixin, Select2MultipleWidget):
    pass

class HeavySelect2MultipleAddAnother(AddAnotherMixin, HeavySelect2MultipleWidget):
    pass

class HeavySelect2TagAddAnother(AddAnotherMixin, HeavySelect2TagWidget):
    pass

class ModelSelect2AddAnother(AddAnotherMixin, ModelSelect2Widget):
    pass

class ModelSelect2MultipleAddAnother(AddAnotherMixin, ModelSelect2MultipleWidget):
    pass

class ModelSelect2TagAddAnother(AddAnotherMixin, ModelSelect2TagWidget):
    pass



from django.http import HttpResponse, Http404
from django.utils.encoding import force_text
import json


class PopupMixin(object):
    """Mixin for :class:`~django.views.generic.edit.CreateView` classes that
    handles the case of the view being opened in an *add another* popup window.
    """
    def form_valid(self, form):
        response = super(PopupMixin, self).form_valid(form)
        if self.is_popup():
            return self.form_valid_popup(form)
        else:
            return response

    def is_popup(self):
        return self.request.GET.get('_popup')

    def post(self, *args, **kwargs):
        if self.is_popup() and 'target_elem' not in self.request.GET:
            raise Http404("'target_elem' GET parameter missing")
        return super(PopupMixin, self).post(*args, **kwargs)

    def form_valid_popup(self, form):
        objinfo = self.get_js_obj_info(form.instance)
        return HttpResponse(self.get_js_template() % json.dumps(objinfo))

    def get_js_template(self):
        return """<script>
          opener.dj_aa_addNewlyCreatedObject(%s);
          window.close();
        </script>
        """

    def get_js_obj_info(self, instance):
        return {
          'target_elem': self.request.GET['target_elem'],
          'pk': instance.pk,
          'label': self.label_from_instance(instance),
        }

    def label_from_instance(self, instance):
        return force_text(instance)



import json

import django
from django import forms
from django.utils.html import escapejs
from django.shortcuts import resolve_url
from django.contrib.staticfiles.templatetags.staticfiles import static


class AddAnotherMixin(object):
    """Mixin for :class:`Widgets <django.forms.widgets.Widget>` classes that
    adds the *add another* button and the JavaScript required for the popup handling.
    """
    class Media:
        js = ['django_addanother/django_addanother.js']

    #: The URL, pattern name or view name to open in the popup. This may be
    #: None iff :meth:`get_add_another_url` is overwritten.
    add_another_url = None

    def __init__(self, *args, **kwargs):
        if 'add_another_url' in kwargs:
            self.add_another_url = kwargs.pop('add_another_url')
        super(AddAnotherMixin, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None, choices=()):
        """Render the parent widget plus the *add another* button.

        This uses ``super().render(...)`` and :meth:`get_add_another_html`.
        """
        base_html = super(AddAnotherMixin, self).render(name, value, attrs, choices)
        return base_html + self.get_add_another_html()

    def get_add_another_html(self):
        """Return the HTML that describes the *add another* button.

        This consists of the JavaScript returned by :meth:`get_popup_opts_js`
        and an ``<a onclick="dj_aa_popup(this)">`` element.
        """
        if django.VERSION < (1, 9):
            add_icon_path = static("admin/img/icon_addlink.gif")
        else:
            add_icon_path = static("admin/img/icon-addlink.svg")
        button_html = '<a href="#" onclick="dj_aa_popup(this); return false"><img src="%s"></a>' % add_icon_path
        return self.get_popup_opts_js() + button_html

    def get_popup_opts_js(self):
        return "<script>var popup_opts = %s</script>" % json.dumps(self.get_popup_opts())

    def get_add_another_url(self):
        """Return the URL of the popup window. This defaults to the :attr:`add_another_url`
        attribute, which is passed to :func:`django.shortcuts.resolve_url`.
        """
        if self.add_another_url is None:
            raise TypeError("Missing 'add_another_url' argument or class attribute")
        return resolve_url(self.add_another_url)

    def get_popup_url(self):
        return self.get_add_another_url() + "?_popup=1&target_elem=" + escapejs(self.attrs['id'])

    def get_popup_size(self):
        """Return the size of the popup window as a ``(width, height)`` tuple.

        Defaults to ``(500, 300)``.
        """
        return (500, 300)

    def get_popup_opts(self):
        w, h = self.get_popup_size()
        url = self.get_popup_url()
        return {
            'width': w,
            'height': h,
            'url': url,
          }


class SelectAddAnother(AddAnotherMixin, forms.Select):
    pass


class SelectMultipleAddAnother(AddAnotherMixin, forms.SelectMultiple):
    pass



# encoding: utf-8

from setuptools import setup


setup(
    name='django_addanother',
    version='0.1.2',
    author='Jonas Haag',
    author_email='jonas@lophus.org',
    packages=['django_addanother'],
    zip_safe=False,
    include_package_data=True,
    url='https://github.com/jonashaag/django-addanother',
    description='"Add another" buttons outside the Django admin',
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: ISC License (ISCL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
    ],
    install_requires=['django'],
)

